@@ -1,8 +1,8 @@
 use v5.10.0;
 
-package JMAP::Tester::Response;
+package JMAP::Tester::Response 0.100;
 # ABSTRACT: what you get in reply to a succesful JMAP request
-$JMAP::Tester::Response::VERSION = '0.026';
+
 use Moo;
 with 'JMAP::Tester::Role::SentenceCollection', 'JMAP::Tester::Role::HTTPResult';
 
@@ -37,13 +37,41 @@ has wrapper_properties => (
 sub items { @{ $_[0]->_items } }
 
 sub add_items {
-  $_[0]->sentence_broker->abort_callback->("can't add items to " . __PACKAGE__);
+  $_[0]->sentence_broker->abort("can't add items to " . __PACKAGE__);
 }
 
-sub sentence_broker {
-  state $BROKER = JMAP::Tester::SentenceBroker->new;
+sub default_diagnostic_dumper {
+  state $default = do {
+    require JSON::MaybeXS;
+    state $json = JSON::MaybeXS->new->utf8->convert_blessed->pretty->canonical;
+    sub { $json->encode($_[0]); }
+  };
+
+  return $default;
 }
 
+has _diagnostic_dumper => (
+  is => 'ro',
+  builder   => 'default_diagnostic_dumper',
+  init_arg  => 'diagnostic_dumper',
+);
+
+sub dump_diagnostic {
+  my ($self, $value) = @_;
+  $self->_diagnostic_dumper->($value);
+}
+
+sub sentence_broker;
+has sentence_broker => (
+  is    => 'ro',
+  lazy  => 1,
+  init_arg => undef,
+  default  => sub {
+    my ($self) = @_;
+    JMAP::Tester::SentenceBroker->new({ response => $self });
+  },
+);
+
 1;
 
 __END__
@@ -58,7 +86,7 @@ JMAP::Tester::Response - what you get in reply to a succesful JMAP request
 
 =head1 VERSION
 
-version 0.026
+version 0.100
 
 =head1 OVERVIEW
 
