@@ -1,8 +1,8 @@
 use v5.10.0;
 use warnings;
-package JMAP::Tester::Role::Result;
+package JMAP::Tester::Role::Result 0.100;
 # ABSTRACT: the kind of thing that you get back for a request
-$JMAP::Tester::Role::Result::VERSION = '0.026';
+
 use Moo::Role;
 
 use JMAP::Tester::Abort ();
@@ -86,7 +86,7 @@ JMAP::Tester::Role::Result - the kind of thing that you get back for a request
 
 =head1 VERSION
 
-version 0.026
+version 0.100
 
 =head1 OVERVIEW
 
